#!/usr/bin/python
# -*- coding: UTF-8 -*-

# Peaustaire is a basic image tool
# Copyright (C)  2006  Christian Mauduit <ufoot@ufoot.org>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software 
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

import sys
import math
import random
import Image

DEBUG=True
RESIZE_ADJUST=1.25

def mix_all_images_randomly(output_file, width, height, input_files):
    """Mixes random images, resizes them and blits them randomly into a single image"""
    if DEBUG:
        print "Creating %dx%d target image" % (width,height)
    output_image=Image.new('RGB',(width,height))
    nb_inputs=len(input_files)
    nb_images_per_side=math.sqrt(nb_inputs)
    nb_images_w=int(nb_images_per_side*math.sqrt(float(width)/height))
    nb_images_h=int(nb_images_per_side*math.sqrt(float(height)/width))
    print (nb_images_w,nb_images_h)
    resize_surface=float(width*height)/(nb_images_w*nb_images_h)
    input_files_tmp=input_files[:]
    for j in range(nb_images_h):
        for i in range(nb_images_w):
            input_file=input_files_tmp[int(random.random()*len(input_files_tmp))]
            if DEBUG:
                print "Processing \"%s\" at position (%d,%d)" % (input_file,i,j)
            input_files_tmp.remove(input_file)
            input_image=Image.open(input_file)
            input_surface=input_image.size[0]*input_image.size[1]
            resize_coef=math.sqrt(resize_surface/input_surface)*RESIZE_ADJUST
            resize_box=map(lambda(x): int(x*resize_coef), input_image.size)
            input_image=input_image.resize(resize_box,Image.BICUBIC)
            blit_point=(int((i+0.5)*(output_image.size[0]/float(nb_images_w))-resize_box[0]/2),int((j+0.5)*(output_image.size[1]/float(nb_images_h))-resize_box[1]/2))
            output_image.paste(input_image,tuple(blit_point))
    if DEBUG:
        print "%d/%d files not treated" % (len(input_files_tmp),len(input_files))
    if DEBUG:
        print "Saving \"%s\"" % output_file
    output_image.save(output_file)
    pass

def usage():
    """Displays command syntax"""
    print "usage: peaustaire <output_file> <width> <height> <file_1> <file_2> ... <file_N>"
    
def main():
    """Main proc, executed when running the script on the command line"""
    if (len(sys.argv)>=5):
        mix_all_images_randomly(sys.argv[1], int(sys.argv[2]), int(sys.argv[3]), sys.argv[4:])
    else:
        usage()

if (__name__) == "__main__":
    main()
